@@ -1,5 +1,5 @@
 #!/bin/bash
-usbhub --disable-i2c --hub D9D1 power state --port 2 --reset
+hubs hackrf_dfu reset
 sleep 1s
 host/build/hackrf-tools/src/hackrf_info
 EXIT_CODE="$?"
