@@ -23,10 +23,12 @@ ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSI
 
 #include "hackrf.h"
 
+#include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
 #ifndef _WIN32
 	#include <unistd.h>
+	#include <signal.h>
 #endif
 #include <libusb.h>
 
@@ -34,15 +36,10 @@ ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSI
 	/* Avoid redefinition of timespec from time.h (included by libusb.h) */
 	#define HAVE_STRUCT_TIMESPEC 1
 	#define strdup               _strdup
+	#define strcasecmp           _stricmp
 #endif
 #include <pthread.h>
 
-#ifndef bool
-typedef int bool;
-	#define true 1
-	#define false 0
-#endif
-
 #ifdef HACKRF_BIG_ENDIAN
 	#define TO_LE(x)     __builtin_bswap32(x)
 	#define TO_LE64(x)   __builtin_bswap64(x)
@@ -59,8 +56,8 @@ typedef int bool;
 // the same values.
 typedef enum {
 	HACKRF_VENDOR_REQUEST_SET_TRANSCEIVER_MODE = 1,
-	HACKRF_VENDOR_REQUEST_MAX2837_WRITE = 2,
-	HACKRF_VENDOR_REQUEST_MAX2837_READ = 3,
+	HACKRF_VENDOR_REQUEST_MAX283X_WRITE = 2,
+	HACKRF_VENDOR_REQUEST_MAX283X_READ = 3,
 	HACKRF_VENDOR_REQUEST_SI5351C_WRITE = 4,
 	HACKRF_VENDOR_REQUEST_SI5351C_READ = 5,
 	HACKRF_VENDOR_REQUEST_SAMPLE_RATE_SET = 6,
@@ -103,6 +100,17 @@ typedef enum {
 	HACKRF_VENDOR_REQUEST_BOARD_REV_READ = 45,
 	HACKRF_VENDOR_REQUEST_SUPPORTED_PLATFORM_READ = 46,
 	HACKRF_VENDOR_REQUEST_SET_LEDS = 47,
+	HACKRF_VENDOR_REQUEST_SET_USER_BIAS_T_OPTS = 48,
+	HACKRF_VENDOR_REQUEST_FPGA_WRITE_REG = 49,
+	HACKRF_VENDOR_REQUEST_FPGA_READ_REG = 50,
+	HACKRF_VENDOR_REQUEST_P2_CTRL = 51,
+	HACKRF_VENDOR_REQUEST_P1_CTRL = 52,
+	HACKRF_VENDOR_REQUEST_SET_NARROWBAND_FILTER = 53,
+	HACKRF_VENDOR_REQUEST_SET_FPGA_BITSTREAM = 54,
+	HACKRF_VENDOR_REQUEST_CLKIN_CTRL = 55,
+	HACKRF_VENDOR_REQUEST_READ_SELFTEST = 56,
+	HACKRF_VENDOR_REQUEST_READ_ADC = 57,
+	HACKRF_VENDOR_REQUEST_TEST_RTC_OSC = 58,
 } hackrf_vendor_request;
 
 #define USB_CONFIG_STANDARD 0x1
@@ -726,6 +734,13 @@ static int hackrf_open_setup(libusb_device_handle* usb_device, hackrf_device** d
 		return HACKRF_ERROR_NO_MEM;
 	}
 
+#if LIBUSB_API_VERSION >= 0x0100010C
+	// WinUSB: Use RAW_IO to improve throughput on RX
+	if (libusb_endpoint_supports_raw_io(usb_device, RX_ENDPOINT_ADDRESS) == 1) {
+		libusb_endpoint_set_raw_io(usb_device, RX_ENDPOINT_ADDRESS, 1);
+	}
+#endif
+
 	lib_device->usb_device = usb_device;
 	lib_device->transfers = NULL;
 	lib_device->callback = NULL;
@@ -858,6 +873,30 @@ int ADDCALL hackrf_device_list_open(
 	return hackrf_open_setup(usb_device, device);
 }
 
+int ADDCALL hackrf_device_list_bus_sharing(hackrf_device_list_t* list, int idx)
+{
+	libusb_device *usb_dev, *hackrf_dev;
+	uint8_t hackrf_bus;
+	int other_device_count = 0;
+	int i;
+	if (list == NULL || list->usb_devices == NULL || list->usb_device_index == NULL ||
+	    idx < 0 || idx > list->devicecount) {
+		return HACKRF_ERROR_INVALID_PARAM;
+	}
+	hackrf_dev = list->usb_devices[list->usb_device_index[idx]];
+	hackrf_bus = libusb_get_bus_number(hackrf_dev);
+	for (i = 0; i < list->usb_devicecount; i++) {
+		usb_dev = (libusb_device*) list->usb_devices[i];
+		// Don't count the HackRF, devices on other buses, or the root hub.
+		if (usb_dev != hackrf_dev &&
+		    libusb_get_bus_number(usb_dev) == hackrf_bus &&
+		    libusb_get_parent(usb_dev) != NULL) {
+			other_device_count++;
+		}
+	}
+	return other_device_count;
+}
+
 int ADDCALL hackrf_set_transceiver_mode(
 	hackrf_device* device,
 	hackrf_transceiver_mode value)
@@ -896,7 +935,36 @@ int ADDCALL hackrf_max2837_read(
 	result = libusb_control_transfer(
 		device->usb_device,
 		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
-		HACKRF_VENDOR_REQUEST_MAX2837_READ,
+		HACKRF_VENDOR_REQUEST_MAX283X_READ,
+		0,
+		register_number,
+		(unsigned char*) value,
+		2,
+		0);
+
+	if (result < 2) {
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	} else {
+		return HACKRF_SUCCESS;
+	}
+}
+
+int ADDCALL hackrf_max2831_read(
+	hackrf_device* device,
+	uint8_t register_number,
+	uint16_t* value)
+{
+	int result;
+
+	if (register_number >= 16) {
+		return HACKRF_ERROR_INVALID_PARAM;
+	}
+
+	result = libusb_control_transfer(
+		device->usb_device,
+		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_MAX283X_READ,
 		0,
 		register_number,
 		(unsigned char*) value,
@@ -929,7 +997,40 @@ int ADDCALL hackrf_max2837_write(
 		device->usb_device,
 		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR |
 			LIBUSB_RECIPIENT_DEVICE,
-		HACKRF_VENDOR_REQUEST_MAX2837_WRITE,
+		HACKRF_VENDOR_REQUEST_MAX283X_WRITE,
+		value,
+		register_number,
+		NULL,
+		0,
+		0);
+
+	if (result != 0) {
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	} else {
+		return HACKRF_SUCCESS;
+	}
+}
+
+int ADDCALL hackrf_max2831_write(
+	hackrf_device* device,
+	uint8_t register_number,
+	uint16_t value)
+{
+	int result;
+
+	if (register_number >= 16) {
+		return HACKRF_ERROR_INVALID_PARAM;
+	}
+	if (value >= 0x4000) {
+		return HACKRF_ERROR_INVALID_PARAM;
+	}
+
+	result = libusb_control_transfer(
+		device->usb_device,
+		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR |
+			LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_MAX283X_WRITE,
 		value,
 		register_number,
 		NULL,
@@ -1092,6 +1193,216 @@ int ADDCALL hackrf_rffc5071_write(
 	}
 }
 
+int ADDCALL hackrf_fpga_read_register(
+	hackrf_device* device,
+	uint8_t register_number,
+	uint8_t* value)
+{
+	USB_API_REQUIRED(device, 0x0109);
+	int result;
+
+	result = libusb_control_transfer(
+		device->usb_device,
+		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_FPGA_READ_REG,
+		0,
+		register_number,
+		(unsigned char*) value,
+		1,
+		0);
+
+	if (result < 1) {
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	} else {
+		return HACKRF_SUCCESS;
+	}
+}
+
+int ADDCALL hackrf_fpga_write_register(
+	hackrf_device* device,
+	uint8_t register_number,
+	uint8_t value)
+{
+	USB_API_REQUIRED(device, 0x0109);
+	int result;
+
+	result = libusb_control_transfer(
+		device->usb_device,
+		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR |
+			LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_FPGA_WRITE_REG,
+		value,
+		register_number,
+		NULL,
+		0,
+		0);
+
+	if (result != 0) {
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	} else {
+		return HACKRF_SUCCESS;
+	}
+}
+
+int ADDCALL hackrf_read_selftest(hackrf_device* device, hackrf_selftest* selftest)
+{
+	USB_API_REQUIRED(device, 0x0109);
+
+	int result;
+
+	result = libusb_control_transfer(
+		device->usb_device,
+		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_READ_SELFTEST,
+		0,
+		0,
+		(unsigned char*) selftest,
+		sizeof(hackrf_selftest),
+		0);
+
+	if (result < 2) {
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	} else {
+		return HACKRF_SUCCESS;
+	}
+}
+
+int ADDCALL hackrf_test_rtc_osc(hackrf_device* device, bool* pass)
+{
+	USB_API_REQUIRED(device, 0x0109);
+
+	int result;
+
+	enum {
+		START_32KHZ_OSCILLATOR,
+		START_FREQ_MONITOR,
+		READ_FREQ_MONITOR,
+		STOP_32KHZ_OSCILLATOR,
+	} step;
+
+	// Enable 32kHz oscillator
+	result = libusb_control_transfer(
+		device->usb_device,
+		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR |
+			LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_TEST_RTC_OSC,
+		0,
+		START_32KHZ_OSCILLATOR,
+		NULL,
+		0,
+		1000);
+
+	if (result < 0) {
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	}
+
+// Wait 1s for oscillator startup
+#ifdef _WIN32
+	Sleep(1000);
+#else
+	usleep(1000000);
+#endif
+
+	// Start frequency monitor
+	result = libusb_control_transfer(
+		device->usb_device,
+		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR |
+			LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_TEST_RTC_OSC,
+		0,
+		START_FREQ_MONITOR,
+		NULL,
+		0,
+		1000);
+
+	if (result < 0) {
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	}
+
+// Wait for frequency monitor result
+#ifdef _WIN32
+	Sleep(1);
+#else
+	usleep(1000);
+#endif
+
+	// Read frequency monitor result
+	uint16_t count = 0;
+	result = libusb_control_transfer(
+		device->usb_device,
+		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_TEST_RTC_OSC,
+		0,
+		READ_FREQ_MONITOR,
+		(unsigned char*) &count,
+		sizeof(count),
+		1000);
+
+	if (result < sizeof(count)) {
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	}
+
+	if (count == 1) {
+		// Disable 32kHz oscillator
+		result = libusb_control_transfer(
+			device->usb_device,
+			LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR |
+				LIBUSB_RECIPIENT_DEVICE,
+			HACKRF_VENDOR_REQUEST_TEST_RTC_OSC,
+			0,
+			STOP_32KHZ_OSCILLATOR,
+			NULL,
+			0,
+			1000);
+
+		if (result < 0) {
+			last_libusb_error = result;
+			return HACKRF_ERROR_LIBUSB;
+		}
+
+		*pass = true;
+	} else {
+		*pass = false;
+	}
+
+	return HACKRF_SUCCESS;
+}
+
+int ADDCALL hackrf_read_adc(hackrf_device* device, uint8_t adc_channel, uint16_t* value)
+{
+	USB_API_REQUIRED(device, 0x0109);
+
+	int result;
+
+	if ((adc_channel & ~0x80) > 7) {
+		return HACKRF_ERROR_INVALID_PARAM;
+	}
+
+	result = libusb_control_transfer(
+		device->usb_device,
+		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_READ_ADC,
+		0,
+		adc_channel,
+		(unsigned char*) value,
+		2,
+		0);
+
+	if (result < 2) {
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	} else {
+		*value = FROM_LE16(*value);
+		return HACKRF_SUCCESS;
+	}
+}
+
 int ADDCALL hackrf_get_m0_state(hackrf_device* device, hackrf_m0_state* state)
 {
 	USB_API_REQUIRED(device, 0x0106)
@@ -1204,7 +1515,7 @@ int ADDCALL hackrf_spiflash_write(
 {
 	int result;
 
-	if (address > 0x0FFFFF) {
+	if ((address + length) > 0x400000) {
 		return HACKRF_ERROR_INVALID_PARAM;
 	}
 
@@ -1235,7 +1546,7 @@ int ADDCALL hackrf_spiflash_read(
 {
 	int result;
 
-	if (address > 0x0FFFFF) {
+	if ((address + length) > 0x400000) {
 		return HACKRF_ERROR_INVALID_PARAM;
 	}
 
@@ -1762,6 +2073,20 @@ static void* transfer_threadproc(void* arg)
 	int error;
 	struct timeval timeout = {0, 500000};
 
+	/*
+	 * hackrf_transfer uses pause() and SIGALRM to print statistics and
+	 * POSIX doesn't specify which thread must recieve the signal, block all
+	 * signals here, so we don't interrupt their reception by
+	 * hackrf_transfer or any other app which uses the library (#1323)
+	 */
+#ifndef _WIN32
+	sigset_t signal_mask;
+	sigfillset(&signal_mask);
+	if (pthread_sigmask(SIG_BLOCK, &signal_mask, NULL) != 0) {
+		return NULL;
+	}
+#endif
+
 	while (device->do_exit == false) {
 		error = libusb_handle_events_timeout(g_libusb_context, &timeout);
 		if ((error != 0) && (error != LIBUSB_ERROR_INTERRUPTED)) {
@@ -2193,6 +2518,9 @@ const char* ADDCALL hackrf_board_id_name(enum hackrf_board_id board_id)
 	case BOARD_ID_HACKRF1_R9:
 		return "HackRF One";
 
+	case BOARD_ID_PRALINE:
+		return "HackRF Pro";
+
 	case BOARD_ID_UNRECOGNIZED:
 		return "unrecognized";
 
@@ -2219,6 +2547,9 @@ extern ADDAPI uint32_t ADDCALL hackrf_board_id_platform(enum hackrf_board_id boa
 	case BOARD_ID_HACKRF1_R9:
 		return HACKRF_PLATFORM_HACKRF1_R9;
 
+	case BOARD_ID_PRALINE:
+		return HACKRF_PLATFORM_PRALINE;
+
 	default:
 		return 0;
 	}
@@ -2897,6 +3228,34 @@ extern ADDAPI const char* ADDCALL hackrf_board_rev_name(enum hackrf_board_rev bo
 	case BOARD_REV_GSG_HACKRF1_R9:
 		return "r9";
 
+	case BOARD_REV_HACKRF1_R10:
+	case BOARD_REV_GSG_HACKRF1_R10:
+		return "r10";
+
+	case BOARD_REV_PRALINE_R0_1:
+	case BOARD_REV_GSG_PRALINE_R0_1:
+		return "r0.1";
+
+	case BOARD_REV_PRALINE_R0_2:
+	case BOARD_REV_GSG_PRALINE_R0_2:
+		return "r0.2";
+
+	case BOARD_REV_PRALINE_R0_3:
+	case BOARD_REV_GSG_PRALINE_R0_3:
+		return "r0.3";
+
+	case BOARD_REV_PRALINE_R1_0:
+	case BOARD_REV_GSG_PRALINE_R1_0:
+		return "r1.0";
+
+	case BOARD_REV_PRALINE_R1_1:
+	case BOARD_REV_GSG_PRALINE_R1_1:
+		return "r1.1";
+
+	case BOARD_REV_PRALINE_R1_2:
+	case BOARD_REV_GSG_PRALINE_R1_2:
+		return "r1.2";
+
 	case BOARD_ID_UNRECOGNIZED:
 		return "unrecognized";
 
@@ -2954,6 +3313,169 @@ int ADDCALL hackrf_set_leds(hackrf_device* device, const uint8_t state)
 	}
 }
 
+int ADDCALL hackrf_set_user_bias_t_opts(
+	hackrf_device* device,
+	hackrf_bias_t_user_settting_req* req)
+{
+	USB_API_REQUIRED(device, 0x0108)
+	uint16_t state = 0; // Assume no modifications
+	if (req->off.do_update) {
+		state |= 0x4;
+		if (req->off.change_on_mode_entry) {
+			state |= 0x2 + req->off.enabled;
+		}
+	}
+
+	if (req->rx.do_update) {
+		state |= 0x20;
+		if (req->rx.change_on_mode_entry) {
+			state |= 0x10 + (req->rx.enabled << 3);
+		}
+	}
+
+	if (req->tx.do_update) {
+		state |= 0x100;
+		if (req->tx.change_on_mode_entry) {
+			state |= 0x80 + (req->tx.enabled << 6);
+		}
+	}
+
+	int result = libusb_control_transfer(
+		device->usb_device,
+		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR |
+			LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_SET_USER_BIAS_T_OPTS,
+		state,
+		0,
+		NULL,
+		0,
+		0);
+
+	if (result != 0) {
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	} else {
+		return HACKRF_SUCCESS;
+	}
+}
+
+int ADDCALL hackrf_set_p1_ctrl(hackrf_device* device, const enum p1_ctrl_signal signal)
+{
+	USB_API_REQUIRED(device, 0x0109);
+
+	int result = libusb_control_transfer(
+		device->usb_device,
+		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR |
+			LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_P1_CTRL,
+		signal,
+		0,
+		NULL,
+		0,
+		0);
+
+	if (result != 0) {
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	} else {
+		return HACKRF_SUCCESS;
+	}
+}
+
+int ADDCALL hackrf_set_p2_ctrl(hackrf_device* device, const enum p2_ctrl_signal signal)
+{
+	USB_API_REQUIRED(device, 0x0109);
+
+	int result = libusb_control_transfer(
+		device->usb_device,
+		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR |
+			LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_P2_CTRL,
+		signal,
+		0,
+		NULL,
+		0,
+		0);
+
+	if (result != 0) {
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	} else {
+		return HACKRF_SUCCESS;
+	}
+}
+
+int ADDCALL hackrf_set_clkin_ctrl(
+	hackrf_device* device,
+	const enum clkin_ctrl_signal signal)
+{
+	USB_API_REQUIRED(device, 0x0109);
+
+	int result = libusb_control_transfer(
+		device->usb_device,
+		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR |
+			LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_CLKIN_CTRL,
+		signal,
+		0,
+		NULL,
+		0,
+		0);
+
+	if (result != 0) {
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	} else {
+		return HACKRF_SUCCESS;
+	}
+}
+
+int ADDCALL hackrf_set_narrowband_filter(hackrf_device* device, const uint8_t value)
+{
+	USB_API_REQUIRED(device, 0x0109);
+
+	int result = libusb_control_transfer(
+		device->usb_device,
+		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR |
+			LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_SET_NARROWBAND_FILTER,
+		value,
+		0,
+		NULL,
+		0,
+		0);
+
+	if (result != 0) {
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	} else {
+		return HACKRF_SUCCESS;
+	}
+}
+
+int ADDCALL hackrf_set_fpga_bitstream(hackrf_device* device, const uint8_t index)
+{
+	USB_API_REQUIRED(device, 0x0109);
+
+	int result = libusb_control_transfer(
+		device->usb_device,
+		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR |
+			LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_SET_FPGA_BITSTREAM,
+		index,
+		0,
+		NULL,
+		0,
+		0);
+
+	if (result != 0) {
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	} else {
+		return HACKRF_SUCCESS;
+	}
+}
+
 #ifdef __cplusplus
 } // __cplusplus defined.
 #endif
